#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
"""

folder_structure = {
                     "data_path" : "data",
                     "grid_path" : "grid",
                     "chem_path" : "chem_thermo_tran"
                 }